/***************************************************************************
                          renderitems.h  -  description
                             -------------------
    begin                : Thu Aug 1 2002
    copyright            : (C) 2002 by Ryan Rusaw
    email                : rrusaw@sasktel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef RENDERITEMS_H
#define RENDERITEMS_H

#include <qcanvas.h>
#include <qobject.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qlist.h>
#include <math.h>
#include <qdict.h>
#include <qxml.h>
#include <qpe/resource.h>
#include <qpe/fontdatabase.h>
#include "pcontent.h"
#include "canvasshapes.h"
#include "pbrowser.h"
#include "putils.h"

class CanvasRoundedRectangle;
class CanvasCutEllipse;


/** \class RenderItems
  * \brief The root render items class.
  * The root class that all other rendered items are derived from.
  * @author Ryan Rusaw
  */
class RenderItems : public QObject
{
    
public:
    /**
      * \brief Constructor.
      * @param parent Parent class.
      * @param canvas Canvas to render to
      * @param contentroot root of content file xml tree
      * @param styleroot root of style file xml tree
      * @param i QList of all the graphic items stored in the .sxi file
      */
    RenderItems(QObject* parent, QCanvas* canvas, pXmlNode *contentroot, pXmlNode*styleroot, QList<pImage> *i);
    /**
      * \brief Deconstructor
      */
    ~RenderItems(){};
    /**
      * \brief Renders to canvas. Reimplemented by descendents.
      * @param node node to render.
      */
    void render(pXmlNode* node);
    /**
      * \brief Generates a gradient image
      * @param gradnode gradient node to generate.
      * @param steps nomber of steps in the gradiant.
      * @param size size of image to create, if size = NULL then canvas size is used.
      * @return QImage of the gradient
      */
    QImage *makeGradientImage(pXmlNode* gradnode, int steps, QSize *size = NULL);
    /**
      * \brief Generates a hatched image
      * @param hatchnode node to generate hatch for,
      * @param color hatching background color.
      * @param tpcy alpha value of hatched background ( either 0x00 or 0xFF )
      * @param size size of image to create, if size = NULL then canvas size is used.
      * @return QImage of the hatching.
      */
    QImage *makeHatchImage(pXmlNode* hatchnode, QColor *color, int tpcy = 0xFF, QSize *size = NULL);
    /**
      * \brief shows the image on the canvas after alpha blending it.
      * @param x x-location on canvas of upper left corner
      * @param y y-location on canvas of upper left corner
      * @param w width of image
      * @param h height of image
      * @param image pointer to QImage to show
      * @param alphamode Mode of alpha blending i. None, flat, gradiant etc. See AlphaMode enum
      * @param alphas Start alpha value
      * @param alphae End alpha value
      */
    void showImage(int x, int y, int w, int h, QImage *image, QString *alphastyle = NULL);
    /**
      * \brief Sets factor
      * @param f value to set factor to.
      */       
    void setFactor(float f){ factor = f; };
    /**
      * \brief Sets layer to 0
      */
    void clearLayers(){ layer = 0; };
    /**
      * \brief Sets replaceItems
      * @param b value to set replaceItems to.
      */
    void setReplace(bool b){ replaceitems = b; };
    /**
      * \brief Sets internal rect value
      * @param r value to set rect to.
      */
    void setRect(QRect *r) { rect = r; };
        
    enum AlphaMode { NoAlphaMode, FlatAlphaMode, RadGradAlphaMode, LinGradAlphaMode, AxGradAlphaMode, EllGradAlphaMode };

protected:
    /**
      * \brief returns a pointer to the canvas to render too.
      * @return QCanvas* to canvas.
      */
    QCanvas *canvas() { return thiscanvas; };
    /**
      * \brief Deprecated alias for findStyleProperty.
      * @see findstylePropertyfindStyleProperty(pXmlNode *node,const QString &val);
      */
    QString findStylePropertyInStyleRecursive(pXmlNode *node,const QString &val);
    /**
      * \brief Finds style property for node.
      * Searches logically through defined nodes and back to master-page-style.
      * @param node node to start search in
      * @param val Property to search for
      * @return value of property, or empty zero length string if not found.
      */
    QString findStyleProperty(pXmlNode *node, const QString &val);
    /**
      * \brief Finds style property for node
      * Searches back up pXmlNode tree till it encounters a node named delimiter. Used in text rendering.
      * @param node node to start search in
      * @param val Property to search for
      * @param delimiter delimiter value
      * @return value of property, or empty zero length string if not found.
      */
    QString findStylePropertyWithParentDelimiter(pXmlNode *node,const QString &val,const QString &delimiter);    
    /**
      * \brief Pointer to rendering canvas
      */
    QCanvas *thiscanvas; 
    /**
      * \brief Pointer to style xml tree
      */
    pXmlNode * style;    
    /**
      * \brief Pointer to content xml tree
      */
    pXmlNode * content;
    /**
      * \brief Pointer to image data
      */
    QList<pImage> images;
    /**
      * \brief Factor to scale the rendered item by.
      */
    float factor;
    /**
      * \brief Internal layer counter for items on canvas
      */
    int layer;
    /**
      * \brief Replaces non implemented items with blank grey box if true
      */
    bool replaceitems;
    /**
      * \brief Internal count of linebreaks in text rendering
      */
    int lbcount;
    /**
      * \brief Internal list incrementer
      */
    bool listinc;
    /**
      * \brief Internal count of last bottom item.
      */
    int lastbottom;
    /**
      * \brief Internal count of text level
      */
    int textlvl;
    /**
      * \brief List of text of current text line
      */
    QList<QCanvasText> currline;
    /**
      * \brief List of current text
      */
    QList<QCanvasText> currtxt;
    /**
      * \brief Node of text indent style definition
      */
    pXmlNode *indentnode;
    /**
      *  \brief Pointer to internal QRect for size overriding.
      */
    QRect *rect;

private:

    void xformRadialGradiant(QImage *image, int steps, int radius, int cx, int cy, QRgb start, QRgb end, bool blendAlpha = false);

    void xformAxialGradiant(QImage *image,int steps, int y2, int mm, int axiald, QRgb start, QRgb stop, bool blendAlpha = false);

    void xformLinearGradiant(QImage *image, int steps, int angle, QRgb start, QRgb end, bool blendAlpha = false);

    void xformEllipsoidGradiant(QImage *image, int steps, int cx, int cy, int angle, int radius, QRgb start, QRgb end, bool blendAlpha = false);

    void xformAlphaChannel(QImage* image, QString *alphastyle);

};

#endif
